unit DepthPeelingMain;

interface

uses
  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, JvExExtCtrls, JvExtComponent, JvPanel, cyBasePanel, cyPanel, Buttons,
  cyBaseSpeedButton, cySpeedButton, dglopengl, CamerAH, StdCtrls;

type

  mMode = (mStandard, mDDP_Color, mDDP_Textur, mDDP_Color_Textur);

  TForm1 = class(TForm)
    tmr1: TTimer;
    pnl1: TPanel;
    cbb1: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure pnl1Resize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tmr1Timer(Sender: TObject);
    procedure cbb1Change(Sender: TObject);
  private
    NearClipping: Single;
    FarClipping: Single;
    Mode: mMode;
    procedure RenderMain;
    procedure SetupGL;
  public
    xDC: Cardinal;
    xRC: Cardinal;
    Counter: Extended;
  end;

var
  Form1: TForm1;
  backgroundColor: TGLVectorf3;

implementation

uses
  Windows, Figures, Math, DepthPeeling, Geometry, glBitmap;

{$R *.dfm}

procedure TForm1.SetupGL;
begin
  NearClipping := 0.0001;
  FarClipping  := 500;
  Counter := 0;
  backgroundColor[0] := 0.3;
  backgroundColor[1] := 0.4;
  backgroundColor[2] := 0.7;
  g_backgroundColor := backgroundColor;

  xDC:= GetDC(pnl1.Handle);

  if not InitOpenGL then Application.Terminate;

  xRC:= CreateRenderingContext( xDC,
                               [opDoubleBuffered],
                               32,
                               24,
                               0,0,0,
                               0);
  ActivateRenderingContext(xDC, xRC);

  glClearColor(backgroundColor[0], backgroundColor[1], backgroundColor[2], 0.0);
  glEnable(GL_DEPTH_TEST);          //Tiefentest aktivieren
  glEnable(GL_CULL_FACE);           //Backface Culling aktivieren
end;

procedure TForm1.cbb1Change(Sender: TObject);
begin
  case cbb1.ItemIndex of
    0:
      begin
        Mode := mStandard;
        glClearColor(backgroundColor[0], backgroundColor[1], backgroundColor[2], 0.0);
        glEnable(GL_DEPTH_TEST);          //Tiefentest aktivieren
        glEnable(GL_CULL_FACE);           //Backface Culling aktivieren
      end;
    1: Mode := mDDP_Color;
    2: Mode := mDDP_Textur;
    3: Mode := mDDP_Color_Textur;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Mode := mDDP_Color;
  SetupGL;

  InitGL;

  Tex := TglBitmap2D.Create('OSB.PNG');
  Tex.GenTexture;
  g_shaderDualPeel_ColorTextur.UseShader;
  g_shaderDualPeel_ColorTextur.bindTexture2D('Texture0', Tex.ID, 0);
  g_shaderDualPeel_ColorTextur.UnUseShader;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  DeactivateRenderingContext;
  DestroyRenderingContext(xRC);
  ReleaseDC(Handle, xDC);
end;

procedure TForm1.pnl1Resize(Sender: TObject);
begin
  reshape;
end;

procedure TForm1.RenderMain;
var
  imageWidth, imageHeight: Integer;
begin
  imageWidth := pnl1.Width;
  imageHeight := pnl1.Height;
  glViewport(0, 0, imageWidth, imageHeight);

  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glEnable(GL_BLEND);
  glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);

  Counter := ifthen(Counter > 360, 0, Counter + 10);
  g_Counter := Counter;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(45.0, imageWidth/imageHeight, NearClipping, FarClipping);

  case Mode of
    mStandard:          Render;
    mDDP_Color:         RenderDualPeeling(0);
    mDDP_Textur:        RenderDualPeeling(1);
    mDDP_Color_Textur:  RenderDualPeeling(2);
  end;

  SwapBuffers(xDC);
end;

procedure TForm1.tmr1Timer(Sender: TObject);
begin
  RenderMain;
end;

end.
