unit Figures;

interface
uses dglopengl, glBitmap;


function DrawSphere(CX, CY, CZ, Radius : TGLFloat; N : Integer;
  shader: GLHandleARB; IsRender: Boolean = true) : Cardinal;

Procedure DrawBoxObject(dx, dy, dz: Single; inverted: boolean;
                        voffset: TGLVectorf3; toffset, tscale: TGLVectorf2);
type
  Vector = array[0..2] of TGLFloat;

Const voffset : TGLVectorf3 = (0.0, 0.0, 0.0);
      toffset : TGLVectorf2 = (0.0, 0.0);
      tscale  : TGLVectorf2 = (1.0, 1.0);

var
  VecArray: array of Vector;
  Tex: TglBitmap2D;

implementation

Procedure DrawBoxObject(dx, dy, dz: Single; inverted: boolean;
                        voffset: TGLVectorf3; toffset, tscale: TGLVectorf2);
Var i : shortint;
Begin
  If (inverted) Then
    i := -1
  Else
    i := +1;

  dx := 0.5 * dx;
  dy := 0.5 * dy;
  dz := 0.5 * dz;

  glBegin(GL_QUADS);
    // left (inverted: right)
    glNormal3f(-1, 0, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx*i + voffset[0], -dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(-dx*i + voffset[0], -dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx*i + voffset[0], +dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx*i + voffset[0], +dy + voffset[1], -dz + voffset[2]);

    // right (inverted: left)
    glNormal3f(1, 0, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(+dx*i + voffset[0], -dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx*i + voffset[0], -dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx*i + voffset[0], +dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx*i + voffset[0], +dy + voffset[1], +dz + voffset[2]);

    // top (inverted: bottom)
    glNormal3f(0, 1, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], +dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], +dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy*i + voffset[1], -dz + voffset[2]);

    // bottom (inverted: top)
    glNormal3f(0, -1, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], -dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], -dy*i + voffset[1], +dz + voffset[2]);

    // front (inverted: back)
    glNormal3f(0, 0, 1);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy + voffset[1], +dz*i + voffset[2]);

    // back (inverted: front)
    glNormal3f(0, 0, -1);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy + voffset[1], -dz*i + voffset[2]);
  glEnd;
End;

function DrawSphere(CX, CY, CZ, Radius : TGLFloat; N : Integer; shader: GLHandleARB;
  IsRender: Boolean = true) : Cardinal;
var
 i,j                  : Integer;
 theta1,theta2,theta3 : TGLFloat;
 X, Y, Z, px, py, pz  : TGLFloat;
begin
  if Radius < 0 then
    Radius :=-Radius;

  if n < 0 then
    n := -n;

  if (n < 4) or (Radius <= 0) then
  begin
    glBegin(GL_POINTS);
    glVertex3f(CX, CY, CZ);
    glEnd;
    exit;
  end;

  SetLength(VecArray, 0);
  for j := 0 to n div 2 -1 do
  begin
    theta1 := J*2*PI/N - PI/2;
    theta2 := (J+1)*2*PI/n - PI/2;
    glBegin(GL_QUAD_STRIP);
      for i := 0 to n do
      begin
        theta3 := i*2*PI/N;
        x := cos(theta2) * cos(theta3);
        y := sin(theta2);
        z := cos(theta2) * sin(theta3);
        px := CX + Radius*x;
        py := CY + Radius*y;
        pz := CZ + Radius*z;

        if IsRender then
        begin
          glNormal3f(X,Y,Z);
          glTexCoord2f(1-I/n, 2*(J+1)/n);
          glVertex3f(px,py,pz);
        end;
        SetLength(VecArray, Length(VecArray) + 1);
        VecArray[Length(VecArray) -1][0] := px;
        VecArray[Length(VecArray) -1][1] := py;
        VecArray[Length(VecArray) -1][2] := pz;

        X := cos(theta1) * cos(theta3);
        Y := sin(theta1);
        Z := cos(theta1) * sin(theta3);
        px := CX + Radius*X;
        py := CY + Radius*Y;
        pz := CZ + Radius*Z;

        if IsRender then
        begin
          glNormal3f(X,Y,Z);
          glTexCoord2f(1-i/n, 2*j/n);
          glVertex3f(px,py,pz);
        end;
        SetLength(VecArray, Length(VecArray) + 1);
        VecArray[Length(VecArray) -1][0] := px;
        VecArray[Length(VecArray) -1][1] := py;
        VecArray[Length(VecArray) -1][2] := pz;
      end;

      if not IsRender then
      begin
        glUniform3fv(glGetUniformLocation(shader, 'Bezier'), Length(VecArray), @VecArray);
      end;

    glEnd;
  end;
end;

end.
