unit ShaderClass;
// Quelle/ Source:
// http://www.delphigl.com/forum/viewtopic.php?p=72141#p72141
// Author1 : DGL-Member Mori
// Author2 : DGL-Member Jens01
/////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes,Dialogs, dglOpenGL, Contnrs;

var
  GL_TEXTURE: array[0..31] of GLenum = (
    GL_TEXTURE0, GL_TEXTURE1, GL_TEXTURE2, GL_TEXTURE3, GL_TEXTURE4, GL_TEXTURE5, GL_TEXTURE6,
    GL_TEXTURE7, GL_TEXTURE8, GL_TEXTURE9, GL_TEXTURE10, GL_TEXTURE11, GL_TEXTURE12, GL_TEXTURE13,
    GL_TEXTURE14, GL_TEXTURE15, GL_TEXTURE16, GL_TEXTURE17, GL_TEXTURE18, GL_TEXTURE19,
    GL_TEXTURE20, GL_TEXTURE21, GL_TEXTURE22, GL_TEXTURE23, GL_TEXTURE24, GL_TEXTURE25,
    GL_TEXTURE26, GL_TEXTURE27, GL_TEXTURE28, GL_TEXTURE29, GL_TEXTURE30, GL_TEXTURE31
    );

const
  GL_NO_ERROR = 0;
  GL_FRAGMENT_ERROR = 1;
  GL_VERTEX_ERROR = 2;
  GL_NO_CODE_LOADED = 3;
  GL_NO_CODE_FOUND = 4;
  GL_COMPILER_ERROR = 5;
  GL_ATTACH_ERROR = 6;
  GL_LINKER_ERROR = 7;
  GL_SHADER_ISNT_READY_TO_USE = 8;
  GL_UNIFORM_ERROR = 9;

type
  TShaderError = GL_NO_ERROR..GL_SHADER_ISNT_READY_TO_USE;
  TShaderErrors = set of TShaderError;

  TOnInternError = procedure(Shader: TObject; ErrorCode: TShaderErrors; ErrorString: String) of Object;

  TVertFrag = class
  private
  protected
    Code: TStringList;
    ShaderID: GLuint;
    FileName: string;
    procedure ResetError;
  public
    IsCodeFound,
    IsCodeCompiled: Boolean;
    ShaderType: GLenum;
    ErrorString: string;
    ErrorCode: TShaderErrors;
    constructor Create;
    destructor Destroy; override;
    procedure Compile;
    procedure Delete;
    function  glSlang_GetShaderInfoLog: String;
    procedure LoadCode(FileName: String);
    procedure SetCode (ACode: TStringList);
  end;

  TShader = class
  private
    ShaderList: TObjectList;
    ProgramID : GLuint;
    IsProgamLinked: Boolean;

    InternErrors: TStringList;

    function glSlang_GetProgramInfoLog(PProgram: GLuint): String;

    function GetShaderHandle: GLuint;
    function GetError(ErrorNr: Integer): String;
    procedure AddError(AErrorCode: TShaderErrors; AErrorString: AnsiString);

    procedure setTextureUnit(texname: AnsiString; texunit: Integer);
    procedure bindTexture(target: GLenum; texname: AnsiString; texid: GLuint; texunit: Integer);
    function  glSlang_GetLinkInfoLog(PProgram: GLuint): AnsiString;
  public
    OnInternError: TOnInternError;
    constructor Create(VertexFile, FragmentFile: String); overload;
    constructor Create(VertexCode, FragmentCode: TStringList); overload;
    constructor Create; overload;
    destructor Destroy;

    procedure AddShaderFile(AShaderType: GLenum; AFileName: String); overload;
    procedure AddShaderFile(AShaderType: GLenum; ACode: TStringList); overload;
    procedure CompileShaders;
    procedure Link;
    procedure CompileAndLink;

    procedure UseShader;
    procedure UnUseShader;

    procedure SetUniformfv(Name: AnsiString; v: PGLfloat; count: Integer);
    procedure SetUniformiv(Name: AnsiString; v: PGLint; count: Integer);
    procedure SetUniformi(Name: AnsiString; v: array of GLint);
    procedure SetUniformf(Name: AnsiString; v: array of GLfloat);
    procedure SetUniformMatrixfv(Name: AnsiString; transpose: GLboolean; v: PGLfloat;
      count: Integer);
    procedure bindTextureRECT(texname: AnsiString; texid: GLuint; texunit: Integer);
    procedure bindTexture2D(texname: AnsiString; texid: GLuint; texunit: Integer);
    procedure bindTexture3D(texname: AnsiString; texid: GLuint; texunit: Integer);

    function  GetAllErrors: String;
    function  GetLastError: String;
    property  ShaderID: GLuint read GetShaderHandle;
    property  Errors [Nr: Integer]: String read GetError;
  end;

  TShaderList = class(TObjectList)
  private
    function  getItem(Index: Integer): TShader; virtual;
    procedure setItem(Index: Integer; Objekt: TShader); virtual;    
  public
    //constructor Create;
    //destructor Destroy; override;
    function  AddandCreate(VertexFile, FragmentFile: String): Integer;
    procedure UseShader(Index: Integer); 
    procedure UnUseShader(Index: Integer); 
  
    
    function  Add(Objekt: TShader): Integer; virtual;
    function  Remove(Objekt: TShader): Integer; virtual;
    function  IndexOf(Objekt: TShader): Integer; virtual;
    procedure Insert(Index: Integer; Objekt: TShader); virtual;
    function  First: TShader; virtual;
    function  Last: TShader; virtual;    

    property Items[Index: Integer]: TShader read getItem write setItem; default;   
  end;

implementation

//********************** TShader *********************

constructor TShader.Create;
begin
  inherited;
  ShaderList := TObjectList.Create;

  InternErrors := TStringList.Create;
  IsProgamLinked := False;
end;

constructor TShader.Create(VertexFile, FragmentFile: String);
begin
  Create;

  AddShaderFile(GL_VERTEX_SHADER, VertexFile);
  AddShaderFile(GL_FRAGMENT_SHADER, FragmentFile);
  CompileShaders;
  Link;
end;

constructor TShader.Create(VertexCode, FragmentCode: TStringList);
begin
  Create;

  AddShaderFile(GL_VERTEX_SHADER, VertexCode);
  AddShaderFile(GL_FRAGMENT_SHADER, FragmentCode);
  CompileShaders;
  Link;
end;

destructor TShader.Destroy;
begin
  InternErrors.Free;
  ShaderList.Free;

  glDeleteProgram(ProgramID);

  inherited;
end;

procedure TShader.CompileAndLink;
begin
  CompileShaders;
  Link;
end;

procedure TShader.CompileShaders;
var
  i: Integer;
begin
  for i := 0 to ShaderList.Count - 1 do
  with TVertFrag(ShaderList[i]) do
  begin
    Compile;
    if ErrorString <> '' then
      AddError(ErrorCode, ErrorString);
  end;
end;

procedure TShader.setTextureUnit(texname: AnsiString; texunit: Integer);
var
  id,
  linked: GLint;
begin
	glGetProgramiv(ProgramID, GL_LINK_STATUS, @linked);
  if not linked = GL_TRUE then
  begin
    AddError([GL_LINKER_ERROR], 'Uniformerror : Shader not linked');
    Exit;
  end;
  id := glGetUniformLocation(ProgramID, PGLChar(texname));
  if id = -1 then
  begin
    //AddError([GL_UNIFORM_ERROR], 'Error: Uniformerror : Variable not found "' +texname+'"');
    //Exit;
  end;
  glUniform1i(id, texunit);
end;

procedure TShader.AddShaderFile(AShaderType: GLenum; AFileName: String);
begin
  ShaderList.Add(TVertFrag.Create);
  with TVertFrag(ShaderList.Last) do
  begin
    ShaderType := AShaderType;
    FileName := AFileName;

    LoadCode(AFileName);

    if ErrorString <> '' then
      AddError(ErrorCode, ErrorString);
  end;
end;

procedure TShader.AddShaderFile(AShaderType: GLenum; ACode: TStringList);
begin
  ShaderList.Add(TVertFrag.Create);
  with TVertFrag(ShaderList.Last) do
  begin
    ShaderType := AShaderType;

    SetCode(ACode);

    if ErrorString <> '' then
      AddError(ErrorCode, ErrorString);
  end;
end;

procedure TShader.bindTexture(target: GLenum; texname: AnsiString; texid: GLuint; texunit: Integer);
begin
	glActiveTexture(GL_TEXTURE[texunit]);
	glBindTexture(target, texid);
	setTextureUnit(texname, texunit);
	glActiveTexture(GL_TEXTURE0);
end;

procedure TShader.bindTextureRECT(texname: AnsiString; texid: GLuint; texunit: Integer);
begin
  bindTexture(GL_TEXTURE_RECTANGLE, texname, texid, texunit);
end;

procedure TShader.bindTexture2D(texname: AnsiString; texid: GLuint; texunit: Integer);
begin
  bindTexture(GL_TEXTURE_2D, texname, texid, texunit);
end;

procedure TShader.bindTexture3D(texname: AnsiString; texid: GLuint; texunit: Integer);
begin
  bindTexture(GL_TEXTURE_3D, texname, texid, texunit);
end;

procedure TShader.SetUniformf(Name: AnsiString; v: array of GLfloat);
var
  count,
  id: GLint;
begin
	id := glGetUniformLocation(ProgramID, PGLChar(name));
  count := Length(v);
  if (id = -1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : Variable not found "' +Name+'"');
    Exit;
  end;
  if (count < 1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
    Exit;
  end;

  case count of
    1: glUniform1f(id, v[0]);
    2: glUniform2f(id, v[0], v[1]);
    3: glUniform3f(id, v[0], v[1], v[2]);
    else
      glUniform4f(id, v[0], v[1], v[2], v[3]);
  end;
end;

procedure TShader.SetUniformi(Name: AnsiString; v: array of GLint);
var
  count,
  id: GLint;
begin
	id := glGetUniformLocation(ProgramID, PGLChar(name));
  count := Length(v);
  if (id = -1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : Variable not found "' +Name+'"');
    Exit;
  end;
  if (count < 1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
    Exit;
  end;

  case count of
    1: glUniform1f(id, v[0]);
    2: glUniform2f(id, v[0], v[1]);
    3: glUniform3f(id, v[0], v[1], v[2]);
    else
      glUniform4f(id, v[0], v[1], v[2], v[3]);
  end;
end;

procedure TShader.SetUniformfv(Name: AnsiString; v: PGLfloat; count: Integer);
var
  id: GLint;
begin
	id := glGetUniformLocation(ProgramID, PGLChar(name));
  if (id = -1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : Variable not found "' +Name+'"');
    Exit;
  end;
  case count of
    1: glUniform1fv(id, 1, v);
    2: glUniform2fv(id, 1, v);
    3: glUniform3fv(id, 1, v);
    4: glUniform4fv(id, 1, v);
    else
      AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
  end;
end;

procedure TShader.SetUniformiv(Name: AnsiString; v: PGLint; count: Integer);
var
  id: GLint;
begin
	id := glGetUniformLocation(ProgramID, PGLChar(name));
  if (id = -1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : Variable not found "' +Name+'"');
    Exit;
  end;
  case count of
    1: glUniform1iv(id, 1, v);
    2: glUniform2iv(id, 1, v);
    3: glUniform3iv(id, 1, v);
    4: glUniform4iv(id, 1, v);
    else
      AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
  end;
end;

procedure TShader.SetUniformMatrixfv(Name: AnsiString; transpose: GLboolean; v: PGLfloat; count: Integer);
var
  id: GLint;
begin
	id := glGetUniformLocation(ProgramID, PGLChar(name));
  if (id = -1) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : Variable not found "' +Name+'"');
    Exit;
  end;
  if (count < 2) then
  begin
    AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
    Exit;
  end;
  case count of
    2: glUniformMatrix2fv(id, 1, transpose, v);
    3: glUniformMatrix3fv(id, 1, transpose, v);
    4: glUniformMatrix4fv(id, 1, transpose, v);
    else
      AddError([GL_UNIFORM_ERROR], 'Uniformerror : No variable send "' +Name+'"');
  end;
end;

function TShader.glSlang_GetLinkInfoLog(PProgram : GLuint) : AnsiString;
var
 blen,slen : GLInt;
 InfoLog   : PGLChar;
 LStatus: GLint;
begin
  Result := '';
  glGetProgramiv(PProgram, GL_LINK_STATUS, @LStatus);

  if LStatus <> GL_TRUE then
  begin
    glGetProgramiv(PProgram, GL_OBJECT_INFO_LOG_LENGTH_ARB , @blen);
    if blen > 1 then
    begin
      GetMem(InfoLog, blen*SizeOf(GLCharARB));
      glGetProgramInfoLog(PProgram, blen, slen, InfoLog);
      Result := PChar(InfoLog);
      Dispose(InfoLog);
    end;
  end;
end;

function TShader.glSlang_GetProgramInfoLog(PProgram : GLuint) : String;
var
 blen,slen : GLInt;
 InfoLog   : PGLChar;
begin
  Result := '';
  glGetProgramiv(PProgram, GL_OBJECT_INFO_LOG_LENGTH_ARB , @blen);

  if blen > 1 then
  begin
    GetMem(InfoLog, blen*SizeOf(GLCharARB));
    glGetProgramInfoLog(PProgram,blen,slen,InfoLog);
    Result  :=  PChar(InfoLog);
    Dispose(InfoLog);
  end;
end;

procedure TShader.Link;
var
  i: Integer;
  ErrStr: Ansistring;
  IsShadersAttached: Boolean;
begin
  IsShadersAttached := False;

  ProgramID := glCreateProgram;
  
  for i := 0 to ShaderList.Count - 1 do
  if not TVertFrag(ShaderList[i]).IsCodeCompiled then
  with TVertFrag(ShaderList[i]) do
  begin
    AddError([GL_LINKER_ERROR], 'Linkerror : One or more shaders are not compiled');
    Exit;
  end;  

  for i := 0 to ShaderList.Count - 1 do
  with TVertFrag(ShaderList[i]) do
  begin
    glAttachShader(ProgramID, ShaderID);
    IsShadersAttached := True;
    Errstr := GlSlang_GetShaderInfoLog;
    Delete;
    if ErrStr <> '' then
    begin
      AddError([GL_ATTACH_ERROR], 'Loaderror : '+#13+#13 + ErrStr);
      IsShadersAttached := False;
      Break;
    end;
  end;

  if IsShadersAttached then
  begin
    glLinkProgram(ProgramID);
    ErrStr := glSlang_GetLinkInfoLog(ProgramID);
    if ErrStr <> '' then
      AddError([GL_LINKER_ERROR], 'Linkerror : '+#13+#13 + ErrStr)
    else
      IsProgamLinked := True;
  end;
end;

procedure TShader.UnUseShader;
begin
  glUseProgram(0);
end;

procedure TShader.UseShader;
begin
  if IsProgamLinked then
    glUseProgram(ProgramID)
  else
  begin
    glUseProgram(0);
    AddError([GL_SHADER_ISNT_READY_TO_USE],'Shader is not ready');
  end;
end;

function TShader.GetAllErrors: String;
var
   i: Integer;
begin
  Result := '';
  for i := 0 to InternErrors.Count-1 do
    Result := Result+Inttostr(i)+': '+InternErrors.Strings[i]+' | ';
end;

function TShader.GetLastError: String;
begin
  if InternErrors.Count > 0 then
   Result := InternErrors[InternErrors.Count-1]
  else
   Result := 'No Error';
end;

function TShader.GetError(ErrorNr: Integer): String;
begin
  result := 'No Error';
  if (InternErrors.Count>ErrorNr) and (0<=ErrorNr) then
   Result := InternErrors.Strings[ErrorNr]
  else
   Result := 'Error existiert nicht';
end;

function TShader.GetShaderHandle: GLuint;
begin
  Result := ProgramID;
end;

procedure TShader.AddError(AErrorCode: TShaderErrors; AErrorString: AnsiString);
begin
  if  (AErrorString  <> '') then
  begin
    if InternErrors.Count = 5 then
      InternErrors.Delete(0);

    InternErrors.Add(AErrorString);

    if Assigned(OnInternError) then
      OnInternError(Self, AErrorCode, GetLastError);

    ShowMessage(AErrorString);
  end;
end;

//********************** TShaderList *********************
procedure TShaderList.UseShader(Index: Integer);
begin
  TShader(Items[Index]).UseShader;
end;

procedure TShaderList.UnUseShader(Index: Integer);
begin
  TShader(Items[Index]).UnUseShader;
end;

function TShaderList.getItem(Index: Integer): TShader;
begin
  Result := TShader(inherited Items[Index]);
end;

procedure TShaderList.setItem(Index: Integer; Objekt: TShader);
begin
  inherited Items[Index] := Objekt;
end;

function TShaderList.Add(Objekt: TShader): Integer;
begin
  Result := inherited Add(Objekt);
end;

function TShaderList.AddandCreate(VertexFile, FragmentFile: String): Integer;
begin
  result := Add(TShader.Create(VertexFile, FragmentFile));
end;

function TShaderList.First: TShader;
begin
  Result := TShader(inherited First());
end;

function TShaderList.IndexOf(Objekt: TShader): Integer;
begin
  Result := inherited IndexOf(Objekt);
end;

procedure TShaderList.Insert(Index: Integer; Objekt: TShader);
begin
  inherited Insert(Index, Objekt);
end;

function TShaderList.Last: TShader;
begin
  Result := TShader(inherited Last());
end;

function TShaderList.Remove(Objekt: TShader): Integer;
begin
  Result := inherited Remove(Objekt);
end; 

//********************** TVertFrag *********************

constructor TVertFrag.Create;
begin
  inherited;
  Code := TSTringList.Create;

  ShaderID := 0;
  IsCodeFound := False;
  IsCodeCompiled := False;
  ResetError;
end;

procedure TVertFrag.Delete;
begin
  glDeleteShader(ShaderID);
end;

destructor TVertFrag.Destroy;
begin
  Code.Free;
  inherited;
end;

procedure TVertFrag.Compile;
var
  C: AnsiString;
  tmpLen: Integer;
  Status: GLint;
begin
  ResetError;
  C := Code.Text;

  if (Length(C) > 0) and IsCodeFound then
  begin
    tmpLen := Length(C);
    ShaderID := glCreateShader(ShaderType);
    glShaderSource(ShaderID, 1, @C, @tmpLen);
    glCompileShader(ShaderID);
    glGetShaderiv(ShaderID, GL_COMPILE_STATUS, @Status);
    if Status = GL_FALSE then
    begin
      ErrorString := ('Shader-Compile-Error  Shader: (' +InttoStr(ShaderID)+  ') ' + FileName +
        #13+#13+GlSlang_GetShaderInfoLog);
      ErrorCode := [GL_COMPILER_ERROR];
      IsCodeCompiled := False;
    end
    else
      IsCodeCompiled := True;
  end
  else
  begin
    ErrorString := ('Shadercode not found '+ FileName);
    ErrorCode := [GL_NO_CODE_FOUND];
    IsCodeFound := False;
  end;
end;

function TVertFrag.glSlang_GetShaderInfoLog: String;
var
 blen, slen : GLInt;
 InfoLog   : PGLChar;
 LStatus: GLint;
begin
  Result := '';
  glGetShaderiv(ShaderID, GL_COMPILE_STATUS, @LStatus);
  if LStatus=GL_FALSE then
  begin
    glGetObjectParameterivARB(ShaderID, GL_OBJECT_INFO_LOG_LENGTH_ARB , @blen);

    if blen > 1 then
    begin
      GetMem(InfoLog, blen*SizeOf(GLCharARB));
      glGetShaderInfoLog(ShaderID,blen,slen,InfoLog);
      Result  :=  PAnsiChar(InfoLog);
      Dispose(InfoLog);
    end;
  end;
end;

procedure TVertFrag.LoadCode;
begin
  ResetError;
  Code.Clear;

  if FileExists(FileName) then
  begin
    Code.LoadFromFile(FileName);
    if Code.Text = '' then
    begin
      ErrorString := ('Error: File "'+FileName+'" not found');
      ErrorCode := [GL_NO_CODE_FOUND];
      IsCodeFound := False;
    end
    else
      IsCodeFound := True;
  end
  else
  begin
    ErrorString := ('Error: File "'+FileName+'" not loaded');
    ErrorCode := [GL_NO_CODE_LOADED];
    IsCodeFound := False;
  end;
end;

procedure TVertFrag.ResetError;
begin
  ErrorString := '';
  ErrorCode   := [GL_NO_ERROR];
end;

procedure TVertFrag.SetCode(ACode: TStringList);
begin
  ResetError;
  Code.Assign(ACode);
  if Code.Text = '' then
  begin
    ErrorString := ('Error: File "'+FileName+'" not found');
    ErrorCode := [GL_NO_CODE_FOUND];
      IsCodeFound := False;
  end
  else
    IsCodeFound := True;
end;

end.
